@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013 The University of Utah
+// Copyright (c) 2012, 2013, 2014 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -173,8 +173,23 @@ bool ClassTemplateToClassSpecializationTypeRewriteVisitor::
   if (TmplKeyLoc.isValid())
     ConsumerInstance->TheRewriter.RemoveText(TmplKeyLoc, 8);
 
-  ConsumerInstance->TheRewriter.RemoveText(SourceRange(Loc.getLAngleLoc(),
-                                                       Loc.getRAngleLoc()));
+  // it's necessary to check the validity of locations, otherwise
+  // we will get assertion errors...
+  // note that some implicit typeloc has been visited by Clang, e.g.
+  // template < typename > struct A { };
+  // struct B:A < int > {
+  //  B (  ) { }
+  // };
+  // base initializer A<int> is not presented in the code but visited,
+  // so, we need to make sure locations are valid.
+  SourceLocation LAngleLoc = Loc.getLAngleLoc();
+  if (LAngleLoc.isInvalid())
+    return true;
+  SourceLocation RAngleLoc = Loc.getRAngleLoc();
+  if (RAngleLoc.isInvalid())
+    return true;
+  ConsumerInstance->TheRewriter.RemoveText(SourceRange(LAngleLoc,
+                                                       RAngleLoc));
   return true;
 }
 
@@ -244,15 +259,21 @@ void ClassTemplateToClass::rewriteClassTemplatePartialSpecs(void)
     const ClassTemplatePartialSpecializationDecl *PartialD = (*I);
     removeTemplateAndParameter(PartialD->getSourceRange().getBegin(), 
                                PartialD->getTemplateParameters());
-
-    TemplateArgumentLoc *ArgLocs = PartialD->getTemplateArgsAsWritten();
-    TransAssert(ArgLocs && "Invalid ArgLocs!");
-    TemplateArgumentLoc FirstArgLoc = ArgLocs[0];
+    
+    const ASTTemplateArgumentListInfo *ArgList = 
+      PartialD->getTemplateArgsAsWritten();
+    TransAssert(ArgList && "Invalid ArgList!");
+    
+    const TemplateArgumentLoc *ArgLocs = ArgList->getTemplateArgs();
+    TransAssert(ArgLocs && "Invalid ArcLogs!");
+    
+    unsigned NumArgs = ArgList->NumTemplateArgs;
+    TransAssert((NumArgs > 0) && "Invalid NumArgs!");
+    
+    const TemplateArgumentLoc FirstArgLoc = ArgLocs[0];
     SourceLocation StartLoc = FirstArgLoc.getSourceRange().getBegin();
 
-    unsigned NumArgs = PartialD->getNumTemplateArgsAsWritten();
-    TransAssert((NumArgs > 0) && "Invalid NumArgs!");
-    TemplateArgumentLoc LastArgLoc = ArgLocs[NumArgs - 1];
+    const TemplateArgumentLoc LastArgLoc = ArgLocs[NumArgs - 1];
     SourceRange LastRange = LastArgLoc.getSourceRange();
     SourceLocation EndLoc = RewriteHelper->getEndLocationUntil(LastRange, '>');
     
