@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013 The University of Utah
+// Copyright (c) 2012, 2013, 2014 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -49,7 +49,8 @@ public:
     : ConsumerInstance(Instance),
       BeingWritten(false),
       BeingAddrTaken(false),
-      BeingIncDec(false)
+      BeingIncDec(false),
+      BeingPartial(false)
   { }
 
   bool VisitVarDecl(VarDecl *VD);
@@ -120,7 +121,7 @@ bool CopyPropCollectionVisitor::VisitBinaryOperator(BinaryOperator *BO)
     return true;
 
   const Expr *Lhs = BO->getLHS()->IgnoreParenCasts();
-  if (!ConsumerInstance->isValidExpr(Lhs))
+  if (!ConsumerInstance->isValidLhs(Lhs))
     return true;
 
   const Expr *Rhs = BO->getRHS()->IgnoreParenCasts();
@@ -293,6 +294,23 @@ bool CopyPropagation::isValidExpr(const Expr *Exp)
   return false;
 }
 
+bool CopyPropagation::isValidLhs(const Expr *Lhs)
+{
+  const Expr *E = Lhs->IgnoreParenCasts();
+
+  switch(E->getStmtClass()) {
+  case Expr::DeclRefExprClass:
+  case Expr::MemberExprClass:
+  case Expr::ArraySubscriptExprClass: // Fall-through
+    return true;
+
+  default:
+    return false;
+  }
+  TransAssert(0 && "Unreachable code!");
+  return false;
+}
+
 bool CopyPropagation::isConstantExpr(const Expr *Exp)
 {
   const Expr *E = Exp->IgnoreParenCasts();
