@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013 The University of Utah
+// Copyright (c) 2012, 2013, 2014 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -20,6 +20,7 @@
 #include "clang/AST/RecordLayout.h"
 #include "clang/AST/RecursiveASTVisitor.h"
 #include "clang/AST/ASTContext.h"
+#include "llvm/ADT/StringRef.h"
 #include "TransformationManager.h"
 
 using namespace clang;
@@ -142,8 +143,12 @@ bool EmptyStructToIntRewriteVisitor::VisitElaboratedTypeLoc(
   }
 
   SourceLocation StartLoc = Loc.getLocStart();
+  if (StartLoc.isInvalid())
+    return true;
   TypeLoc TyLoc = Loc.getNamedTypeLoc();
   SourceLocation EndLoc = TyLoc.getLocStart();
+  if (EndLoc.isInvalid())
+    return true;
   EndLoc = EndLoc.getLocWithOffset(-1);
   const char *StartBuf = 
     ConsumerInstance->SrcManager->getCharacterData(StartLoc);
@@ -161,9 +166,10 @@ bool EmptyStructToIntRewriteVisitor::VisitElaboratedTypeLoc(
   // We need to omit it.
   if (StartBuf > EndBuf) {
     SourceLocation KeywordLoc = Loc.getElaboratedKeywordLoc();
-    const char *Keyword = TypeWithKeyword::getKeywordName(ETy->getKeyword());
+    const llvm::StringRef Keyword = 
+      TypeWithKeyword::getKeywordName(ETy->getKeyword());
     ConsumerInstance->TheRewriter.ReplaceText(KeywordLoc, 
-                                              strlen(Keyword), "int");
+                                              Keyword.size(), "int");
     return true;
   }
   
@@ -300,6 +306,9 @@ bool EmptyStructToInt::isValidRecordDecl(const RecordDecl *RD)
       return true;
     }
     else {
+      // skip invalid decl, which causes clang assertion errors
+      if (Def->isInvalidDecl())
+        return false;
       // handle another special case where a struct has an unreferenced
       // field. In some cases, we cannot simply remove this field
       // because an empty struct would make a bug disappear.
