@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012 The University of Utah
+// Copyright (c) 2012, 2013, 2014 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -14,6 +14,7 @@
 
 #include "ParamToGlobal.h"
 
+#include "clang/AST/ASTContext.h"
 #include "clang/Basic/SourceManager.h"
 
 #include "TransformationManager.h"
@@ -182,6 +183,11 @@ bool ParamToGlobal::isValidFuncDecl(FunctionDecl *FD)
 
   for (FunctionDecl::param_const_iterator PI = FD->param_begin(),
        PE = FD->param_end(); PI != PE; ++PI) {
+    if ((*PI)->isImplicit() || (*PI)->getSourceRange().isInvalid())
+      continue;
+    if (!FD->hasBody() && (*PI)->getNameAsString().empty())
+      continue;
+
     ValidInstanceNum++;
 
     if (ValidInstanceNum == TransformationCounter) {
@@ -197,10 +203,7 @@ bool ParamToGlobal::isValidFuncDecl(FunctionDecl *FD)
 
 ParamToGlobal::~ParamToGlobal(void)
 {
-  if (CollectionVisitor)
-    delete CollectionVisitor;
-
-  if (RewriteVisitor)
-    delete RewriteVisitor;
+  delete CollectionVisitor;
+  delete RewriteVisitor;
 }
 
