@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012 The University of Utah
+// Copyright (c) 2012, 2014 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -167,7 +167,7 @@ void ReduceArraySize::rewriteArrayVarDecl(void)
   unsigned int Dim = getArrayDimension(ArrayTy);
   TransAssert((Dim > TheDimIdx) && "Bad Dimension Index!");
 
-  const VarDecl *FirstVD = TheVarDecl->getFirstDeclaration();
+  const VarDecl *FirstVD = TheVarDecl->getCanonicalDecl();
   for(VarDecl::redecl_iterator RI = FirstVD->redecls_begin(),
       RE = FirstVD->redecls_end(); RI != RE; ++RI) {
     BracketLocPair LocPair;
