@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013 The University of Utah
+// Copyright (c) 2012, 2013, 2014 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -164,7 +164,13 @@ bool PointerLevelCollectionVisitor::VisitDeclaratorDecl(DeclaratorDecl *DD)
 
 bool PointerLevelCollectionVisitor::VisitUnaryOperator(UnaryOperator *UO)
 {
-  if (UO->getOpcode() != UO_AddrOf)
+  UnaryOperator::Opcode Op = UO->getOpcode();
+  if (Op == UO_Deref) {
+    ConsumerInstance->checkPrefixAndPostfix(UO);
+    return true;
+  }
+
+  if (Op != UO_AddrOf)
     return true;
 
   const Expr *SubE = UO->getSubExpr()->IgnoreParenCasts();
@@ -303,7 +309,8 @@ bool PointerLevelRewriteVisitor::VisitBinaryOperator(BinaryOperator *BO)
       dyn_cast<CXXUnresolvedConstructExpr>(Lhs))
     return true;
   const DeclaratorDecl *DD = ConsumerInstance->getRefDecl(Lhs);
-  TransAssert(DD && "NULL DD!");
+  // it's not always we could have a nonnull DD
+  // TransAssert(DD && "NULL DD!");
   if (DD != ConsumerInstance->TheDecl)
     return true;
 
@@ -363,6 +370,8 @@ bool PointerLevelRewriteVisitor::VisitDeclRefExpr(DeclRefExpr *DRE)
 
   if ((DD == ConsumerInstance->TheDecl) &&
      !(ConsumerInstance->VisitedDeclRefExprs.count(DRE))) {
+    // FIXME: handle cases below
+    // int foo(void) { int *a; return a[0]; }
     ConsumerInstance->rewriteDeclRefExpr(DRE);
   }
 
@@ -523,7 +532,7 @@ const DeclRefExpr *ReducePointerLevel::getDeclRefExpr(const Expr *Exp)
   UnaryOperator::Opcode Op = UO->getOpcode();
   (void)Op;
   TransAssert(((Op == UO_Deref) || (Op == UO_AddrOf)) && 
-              "Non-Deref-or-AddrOf Opcode!");
+              "Invalid Unary Opcode!");
   const Expr *SubE = UO->getSubExpr();
   return getDeclRefExpr(SubE);
 }
@@ -549,12 +558,27 @@ const DeclaratorDecl *ReducePointerLevel::getRefDecl(const Expr *Exp)
 
   UnaryOperator::Opcode Op = UO->getOpcode();
   (void)Op;
-  TransAssert(((Op == UO_Deref) || (Op == UO_AddrOf)) && 
-              "Non-Deref-or-AddrOf Opcode!");
+  TransAssert(((Op == UO_Deref) || (Op == UO_AddrOf) ||
+              UO->isPrefix() || UO->isPostfix()) && 
+              "Invalid Unary Opcode!");
   const Expr *SubE = UO->getSubExpr();
   return getRefDecl(SubE);
 }
 
+void ReducePointerLevel::checkPrefixAndPostfix(const UnaryOperator *UO)
+{
+  const Expr *SubE = UO->getSubExpr()->IgnoreParenCasts();
+  const UnaryOperator *SubUO = dyn_cast<UnaryOperator>(SubE);
+  if (!SubUO)
+    return;
+  if (!SubUO->isPrefix() && !SubUO->isPostfix())
+    return;
+  const DeclaratorDecl *DD = getRefDecl(SubUO->getSubExpr());
+  if (DD) {
+    ValidDecls.erase(DD);
+  }
+}
+
 void ReducePointerLevel::addOneDecl(const DeclaratorDecl *DD, 
                                     int IndirectLevel)
 {
@@ -743,6 +767,11 @@ void ReducePointerLevel::getNewGlobalInitStr(const Expr *Init,
     return;
   }
 
+  case Expr::CXXNewExprClass: {
+    InitStr = "";
+    return;
+  }
+
   default:
     TransAssert(0 && "Uncatched initializer!");
   }
@@ -760,6 +789,15 @@ void ReducePointerLevel::getNewLocalInitStr(const Expr *Init,
     RewriteHelper->getExprString(E, InitStr);
     return;
 
+  // FIXME: not quite correct to set the InitStr to an empty string
+  case Expr::GNUNullExprClass:
+    InitStr = "";
+    return;
+
+  case Expr::CXXNewExprClass:
+    InitStr = "";
+    return;
+
   case Expr::StringLiteralClass:
     InitStr = 'a';
     return;
@@ -816,6 +854,8 @@ void ReducePointerLevel::getNewLocalInitStr(const Expr *Init,
 
   case Expr::MemberExprClass: // Fall-through
   case Expr::BinaryOperatorClass: 
+  case Expr::CXXMemberCallExprClass: 
+  case Expr::CallExprClass: 
   case Expr::ArraySubscriptExprClass: {
     RewriteHelper->getExprString(E, InitStr);
     InitStr = "*(" + InitStr + ")";
