@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013 The University of Utah
+// Copyright (c) 2012, 2013, 2014 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -145,6 +145,13 @@ void RemoveBaseClass::handleOneCXXRecordDecl(const CXXRecordDecl *CXXRD)
     const CXXRecordDecl *Base = NULL;
     for (CXXRecordDeclSet::iterator I = AllBaseClasses.begin(), 
          E = AllBaseClasses.end(); I != E; ++I) {
+      if (const ClassTemplateSpecializationDecl * CTSD = 
+          dyn_cast<ClassTemplateSpecializationDecl>
+            (CanonicalRD->getDefinition())) {
+        if (!CTSD->isExplicitSpecialization())
+          continue;
+      }
+
       if (isDirectlyDerivedFrom(CanonicalRD, *I)) {
         Base = (*I);
         ValidInstanceNum++;
@@ -283,7 +290,7 @@ void RemoveBaseClass::removeBaseInitializer(void)
 {
   for (CXXRecordDecl::ctor_iterator I = TheDerivedClass->ctor_begin(),
        E = TheDerivedClass->ctor_end(); I != E; ++I) {
-    if ((*I)->isThisDeclarationADefinition() && !(*I)->isImplicitlyDefined())
+    if ((*I)->isThisDeclarationADefinition() && !(*I)->isDefaulted())
       rewriteOneCtor(*I);
   }
 }
