@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013 The University of Utah
+// Copyright (c) 2012, 2013, 2014 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -216,6 +216,11 @@ bool RemoveNestedFunction::writeNewTmpVariable(const QualType &QT,
   return RewriteHelper->addLocalVarToFunc(VarStr, TheFuncDecl);
 }
 
+bool RemoveNestedFunction::writeNewIntTmpVariable(std::string &VarStr)
+{
+  return RewriteHelper->addLocalVarToFunc("int " + VarStr + ";", TheFuncDecl);
+}
+
 bool RemoveNestedFunction::addNewTmpVariable(ASTContext &ASTCtx)
 {
   std::string VarStr;
@@ -240,27 +245,57 @@ bool RemoveNestedFunction::addNewTmpVariable(ASTContext &ASTCtx)
     // In this case, we have to lookup a corresponding function decl
 
     DeclarationName DName = UE->getName();
-    TransAssert((DName.getNameKind() == DeclarationName::Identifier) &&
+    TransAssert(((DName.getNameKind() == DeclarationName::Identifier) ||
+                 (DName.getNameKind() == DeclarationName::CXXOperatorName)) &&
                 "Not an indentifier!");
     const FunctionDecl *FD = NULL;
     if (const NestedNameSpecifier *NNS = UE->getQualifier()) {
-      if (const DeclContext *Ctx = getDeclContextFromSpecifier(NNS))
-        FD = lookupFunctionDecl(DName, Ctx);
+      if (const DeclContext *Ctx = getDeclContextFromSpecifier(NNS)) {
+        DeclContextSet VisitedCtxs;
+        FD = lookupFunctionDecl(DName, Ctx, VisitedCtxs);
+      }
     }
+    if (!FD) {
+      DeclContextSet VisitedCtxs;
+      FD = 
+        lookupFunctionDecl(DName, TheFuncDecl->getLookupParent(), VisitedCtxs);
+    }
+    // give up and generate a tmp var of int type, e.g.:
+    // template <class T> struct S {
+    //   T x;
+    //   template <class A> void foo(A &a0) { x(y(a0)); } 
+    // };
     if (!FD)
-      FD = lookupFunctionDecl(DName, TheFuncDecl->getLookupParent());
-    TransAssert(FD && "Cannot resolve DName!");
-    QT = FD->getResultType();
+      return writeNewIntTmpVariable(VarStr);
+
+    QT = FD->getReturnType();
+    //FIXME: This is actually not quite correct, we should get the instantiated
+    // type here.
     return writeNewTmpVariable(QT, VarStr);
   }
 
   if (const UnresolvedMemberExpr *UM = dyn_cast<UnresolvedMemberExpr>(E)) {
     DeclarationName DName = UM->getMemberName();
     CXXRecordDecl *CXXRD = UM->getNamingClass();
-    const FunctionDecl *FD = lookupFunctionDecl(DName, CXXRD);
+    DeclContextSet VisitedCtxs;
+    const FunctionDecl *FD = lookupFunctionDecl(DName, CXXRD, VisitedCtxs);
     // FIXME: try to resolve FD here
     if (FD)
-      QT = FD->getResultType();
+      QT = FD->getReturnType();
+    return writeNewTmpVariable(QT, VarStr);
+  }
+
+  if (const CXXTemporaryObjectExpr *CXXTE = 
+      dyn_cast<CXXTemporaryObjectExpr>(E)) {
+    const CXXConstructorDecl *CXXCtor = CXXTE->getConstructor();
+    QT = CXXCtor->getThisType(ASTCtx);
+    return writeNewTmpVariable(QT, VarStr);
+  }
+
+  if (const CXXTemporaryObjectExpr *CXXTE = 
+      dyn_cast<CXXTemporaryObjectExpr>(E)) {
+    const CXXConstructorDecl *CXXCtor = CXXTE->getConstructor();
+    QT = CXXCtor->getThisType(ASTCtx);
     return writeNewTmpVariable(QT, VarStr);
   }
 
@@ -292,9 +327,10 @@ bool RemoveNestedFunction::addNewTmpVariable(ASTContext &ASTCtx)
     if (dyn_cast<CXXThisExpr>(BaseE)) {
       const DeclContext *Ctx = TheFuncDecl->getLookupParent();
       TransAssert(Ctx && "Bad DeclContext!");
-      const FunctionDecl *FD = lookupFunctionDecl(DName, Ctx);
+      DeclContextSet VisitedCtxs;
+      const FunctionDecl *FD = lookupFunctionDecl(DName, Ctx, VisitedCtxs);
       TransAssert(FD && "Cannot resolve DName!");
-      QT = FD->getResultType();
+      QT = FD->getReturnType();
       return writeNewTmpVariable(QT, VarStr);
     }
     
@@ -302,11 +338,12 @@ bool RemoveNestedFunction::addNewTmpVariable(ASTContext &ASTCtx)
     // the current CXXRecord, e.g.,
     const Type *Ty = ME->getBaseType().getTypePtr();
     if (const DeclContext *Ctx = getBaseDeclFromType(Ty)) {
-      const FunctionDecl *FD = lookupFunctionDecl(DName, Ctx);
+      DeclContextSet VisitedCtxs;
+      const FunctionDecl *FD = lookupFunctionDecl(DName, Ctx, VisitedCtxs);
       if (!FD) {
         return writeNewTmpVariable(QT, VarStr);
       }
-      QT = FD->getResultType();
+      QT = FD->getReturnType();
       const Type *RVTy = QT.getTypePtr();
       if (RVTy->getAs<InjectedClassNameType>()) {
         // handle cases like:
@@ -357,7 +394,10 @@ bool RemoveNestedFunction::addNewTmpVariable(ASTContext &ASTCtx)
       // In this case, G[0] is of BuiltinType.
       // But why does clang represent a dependent type as BuiltinType here?
 
-      TransAssert((Ty->getAs<BuiltinType>() || Ty->getAs<TemplateTypeParmType>())
+      TransAssert((Ty->getAs<BuiltinType>() || 
+                   Ty->getAs<TemplateTypeParmType>() ||
+                   Ty->getAs<TypedefType>() ||
+                   Ty->getAs<DependentNameType>())
                   && "Uncaught Type");
       // FIXME: This is incorrect!
       // a couple of questions
@@ -370,10 +410,23 @@ bool RemoveNestedFunction::addNewTmpVariable(ASTContext &ASTCtx)
       // DependentTy and users should never see it; however, it is here to
       // help diagnose failures to properly check for type-dependent
       // expressions.
-      return writeNewTmpVariable(QT, VarStr);
+      return writeNewIntTmpVariable(VarStr);
     }
   }
 
+  const Type *CalleeType = E->getType().getTypePtr();
+  // template <class T1, class T2> struct S {
+  //   T1 x; T2 y;
+  //   template <class A> void foo(A &a0) { x(y(a0)); }
+  // };
+  if (const TemplateTypeParmType *PT = 
+      dyn_cast<TemplateTypeParmType>(CalleeType)) {
+    const TemplateTypeParmDecl *PD = PT->getDecl();
+    std::string DStr = PD->getNameAsString();
+    VarStr = DStr + " " + VarStr + ";";
+    return RewriteHelper->addLocalVarToFunc(VarStr, TheFuncDecl);
+  }
+  //  return writeNewIntTmpVariable(VarStr);
   QT = TheCallExpr->getCallReturnType();
   return writeNewTmpVariable(QT, VarStr);
 }
