@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013 The University of Utah
+// Copyright (c) 2012, 2013, 2014 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -432,13 +432,15 @@ bool RenameCXXMethod::isValidName(const StringRef &Name)
 }
 
 void RenameCXXMethod::addOneMethodName(const CXXMethodDecl *MD,
-                                       unsigned int Num)
+                                       unsigned int /*Num*/)
 {
   const CXXMethodDecl *CanonicalMD = MD->getCanonicalDecl();
   TransAssert((NewMethodNames.find(CanonicalMD) == NewMethodNames.end()) &&
               "Duplicate CXXMethodDecl!");
   std::stringstream SS;
-  SS << MethodNamePrefix << Num;
+  // SS << MethodNamePrefix << Num;
+  NumRenamedMethods++;
+  SS << MethodNamePrefix << NumRenamedMethods;
   NewMethodNames[CanonicalMD] = SS.str();
 
   // Now we check if the old name actually has a valid format, i.e. m_fn([0-9]+)
@@ -608,7 +610,8 @@ const FunctionDecl* RenameCXXMethod::getFunctionDeclFromOverloadExpr(
   DeclarationName DName = OE->getName();
   TransAssert((DName.getNameKind() == DeclarationName::Identifier) &&
               "Not an indentifier!"); 
-  return lookupFunctionDecl(DName, RD);
+  DeclContextSet VisitedCtxs;
+  return lookupFunctionDecl(DName, RD, VisitedCtxs);
 }
 
 const FunctionDecl* RenameCXXMethod::getFunctionDeclFromType(
@@ -618,7 +621,8 @@ const FunctionDecl* RenameCXXMethod::getFunctionDeclFromType(
     Ty = getBasePointerElemType(Ty);
   const FunctionDecl *FD = NULL;
   if (const CXXRecordDecl *BaseRD = getBaseDeclFromType(Ty)) {
-    FD = lookupFunctionDecl(DName, BaseRD);
+    DeclContextSet VisitedCtxs;
+    FD = lookupFunctionDecl(DName, BaseRD, VisitedCtxs);
   }
   return FD;
 }
@@ -633,12 +637,13 @@ const FunctionDecl* RenameCXXMethod::getFunctionDeclFromOverloadTemplate(
     const DeclContext *Ctx = CurrentFD->getLookupParent();
     TransAssert(Ctx && "Bad DeclContext!");
     DeclarationName FunName = OE->getName();
-    FD = lookupFunctionDecl(FunName, Ctx);
+    DeclContextSet VisitedCtxs;
+    FD = lookupFunctionDecl(FunName, Ctx, VisitedCtxs);
     if (!FD)
       return NULL;
   }
 
-  const Type *Ty = FD->getResultType().getTypePtr();
+  const Type *Ty = FD->getReturnType().getTypePtr();
   return getFunctionDeclFromType(Ty, DName);
 }
 
@@ -663,7 +668,7 @@ const FunctionDecl* RenameCXXMethod::getFunctionDeclFromReturnType(
     const CXXMethodDecl *MD = dyn_cast<CXXMethodDecl>(VD);
     if (!MD)
       return NULL;
-    const Type *Ty = MD->getResultType().getTypePtr();
+    const Type *Ty = MD->getReturnType().getTypePtr();
     return getFunctionDeclFromType(Ty, DName);
   }
   else if (const DeclRefExpr *DRE = dyn_cast<DeclRefExpr>(CalleeExpr)) {
@@ -671,7 +676,7 @@ const FunctionDecl* RenameCXXMethod::getFunctionDeclFromReturnType(
     const CXXMethodDecl *MD = dyn_cast<CXXMethodDecl>(VD);
     if (!MD)
       return NULL;
-    const Type *Ty = MD->getResultType().getTypePtr();
+    const Type *Ty = MD->getReturnType().getTypePtr();
     return getFunctionDeclFromType(Ty, DName);
   }
   else if (const OverloadExpr *OE = dyn_cast<OverloadExpr>(CalleeExpr)) {
@@ -699,7 +704,7 @@ const FunctionDecl* RenameCXXMethod::getFunctionDeclFromReturnType(
     const FunctionDecl *FD = getFunctionDecl(DE);
     if (!FD)
       return NULL;
-    const Type *Ty = FD->getResultType().getTypePtr();
+    const Type *Ty = FD->getReturnType().getTypePtr();
     // Note that it's not always true that we could get a 
     // non-null function here, e.g.:
     // template <typename T> class A {
@@ -729,6 +734,9 @@ const FunctionDecl* RenameCXXMethod::getFunctionDecl(
   if (DE->isImplicitAccess())
     return NULL;
   DeclarationName DName = DE->getMember();
+  if (DName.getNameKind() == DeclarationName::CXXOperatorName)
+    return NULL;
+
   TransAssert((DName.getNameKind() == DeclarationName::Identifier) &&
                 "Not an indentifier!");
   const Expr *E = DE->getBase();
@@ -740,7 +748,8 @@ const FunctionDecl* RenameCXXMethod::getFunctionDecl(
     TransAssert(CurrentFD && "NULL CurrentFD!");
     const DeclContext *Ctx = CurrentFD->getLookupParent();
     TransAssert(Ctx && "Bad DeclContext!");
-    FD = lookupFunctionDecl(DName, Ctx);
+    DeclContextSet VisitedCtxs;
+    FD = lookupFunctionDecl(DName, Ctx, VisitedCtxs);
     TransAssert(FD && "Cannot resolve DName!");
     return FD;
   }
@@ -773,7 +782,8 @@ const FunctionDecl* RenameCXXMethod::getFunctionDecl(
   if (Ty->isPointerType() || Ty->isReferenceType())
     Ty = getBasePointerElemType(Ty);
   if (const DeclContext *Ctx = getBaseDeclFromType(Ty)) {
-    return lookupFunctionDecl(DName, Ctx);
+    DeclContextSet VisitedCtxs;
+    return lookupFunctionDecl(DName, Ctx, VisitedCtxs);
   }
   return NULL;
 }
