@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012 The University of Utah
+// Copyright (c) 2012, 2013, 2014 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -17,6 +17,7 @@
 #include <sstream>
 
 #include "clang/AST/RecursiveASTVisitor.h"
+#include "clang/AST/ASTContext.h"
 #include "clang/Basic/SourceManager.h"
 
 #include "TransformationManager.h"
@@ -36,8 +37,7 @@ class ExistingVarCollectionVisitor : public
 public:
 
   explicit ExistingVarCollectionVisitor(RenameParam *Instance)
-    : ConsumerInstance(Instance),
-      TheFunctionDecl(NULL)
+    : ConsumerInstance(Instance)
   { }
 
   bool VisitVarDecl(VarDecl *VD);
@@ -45,9 +45,6 @@ public:
 private:
 
   RenameParam *ConsumerInstance;
-
-  FunctionDecl *TheFunctionDecl;
-
 };
 
 class RenameParamVisitor : public RecursiveASTVisitor<RenameParamVisitor> {
@@ -141,16 +138,9 @@ void RenameParam::Initialize(ASTContext &context)
   ValidInstanceNum = 1;
 }
 
-bool RenameParam::HandleTopLevelDecl(DeclGroupRef D) 
-{
-  for (DeclGroupRef::iterator I = D.begin(), E = D.end(); I != E; ++I) {
-    VarCollectionVisitor->TraverseDecl(*I);
-  }
-  return true;
-}
-
 void RenameParam::HandleTranslationUnit(ASTContext &Ctx)
 {
+  VarCollectionVisitor->TraverseDecl(Ctx.getTranslationUnitDecl());
   if (QueryInstanceOnly) {
     if (!HasValidParams)
       ValidInstanceNum = 0;
