@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013 The University of Utah
+// Copyright (c) 2012, 2013, 2014 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -147,7 +147,13 @@ bool ReplaceCallExprVisitor::VisitCallExpr(CallExpr *CE)
   if (!FD)
     return true;
 
-  const Type *T = CE->getCallReturnType().getTypePtr();
+  const Type *T;
+  // Because CE->getCallReturnType() fails on builtin functions,
+  // try to get returntype from FD (probably not really accurate thought)
+  if (FD->getBuiltinID())
+    T = FD->getReturnType().getTypePtr();
+  else 
+    T = CE->getCallReturnType().getTypePtr();
   if (T->isVoidType())
     return true;
 
