@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013 The University of Utah
+// Copyright (c) 2012, 2013, 2014 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -103,7 +103,7 @@ bool ReturnVoid::isNonVoidReturnFunction(FunctionDecl *FD)
   // then the type source info won't be available, let's try to
   // get one from the one which is in the source
   if (!FD->getTypeSourceInfo()) {
-    const FunctionDecl *FirstFD = FD->getFirstDeclaration();
+    const FunctionDecl *FirstFD = FD->getCanonicalDecl();
     FD = NULL;
     for (FunctionDecl::redecl_iterator I = FirstFD->redecls_begin(), 
          E = FirstFD->redecls_end(); I != E; ++I) {
@@ -119,7 +119,7 @@ bool ReturnVoid::isNonVoidReturnFunction(FunctionDecl *FD)
   SourceLocation SLoc = TLoc.getBeginLoc();
   if (SLoc.isInvalid())
     return false;
-  QualType RVType = FD->getResultType();
+  QualType RVType = FD->getReturnType();
   return !(RVType.getTypePtr()->isVoidType());
 }
 
