@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013 The University of Utah
+// Copyright (c) 2012, 2013, 2014 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -616,14 +616,42 @@ bool RewriteUtils::getStmtString(const Stmt *S,
   return true;
 }
 
+SourceLocation RewriteUtils::getExpansionEndLoc(SourceLocation EndLoc)
+{
+    FileID FID = SrcManager->getFileID(EndLoc);
+    const SrcMgr::SLocEntry *Entry = &SrcManager->getSLocEntry(FID);
+
+    while (Entry->getExpansion().getExpansionLocStart().isMacroID()) {
+      EndLoc = Entry->getExpansion().getExpansionLocStart();
+      FID = SrcManager->getFileID(EndLoc);
+      Entry = &SrcManager->getSLocEntry(FID);
+    }
+
+    return Entry->getExpansion().getExpansionLocEnd();
+}
+
 bool RewriteUtils::replaceExpr(const Expr *E, 
                                const std::string &ES)
 {
   SourceRange ExprRange = E->getSourceRange();
    
   int RangeSize = TheRewriter->getRangeSize(ExprRange);
-  if (RangeSize == -1)
-    return false;
+  if (RangeSize == -1) {
+    SourceLocation StartLoc = ExprRange.getBegin();
+    if (!SrcManager->isMacroBodyExpansion(StartLoc))
+      return false;
+    StartLoc = SrcManager->getFileLoc(StartLoc);
+    SourceLocation EndLoc = ExprRange.getEnd();
+    if (!SrcManager->isMacroBodyExpansion(EndLoc))
+      return false;
+
+    // FIXME: handle cases below:
+    // #define macro bar(1,2);
+    // int bar(int p1, int p2) { return p1 + p2; }
+    // void foo(void) { int x = macro }
+    EndLoc = getExpansionEndLoc(EndLoc);
+    return !(TheRewriter->ReplaceText(SourceRange(StartLoc, EndLoc), ES));
+  }
 
   return !(TheRewriter->ReplaceText(ExprRange, ES));
 }
@@ -645,10 +673,14 @@ bool RewriteUtils::replaceExprNotInclude(const Expr *E,
 }
 
 std::string RewriteUtils::getStmtIndentString(Stmt *S,
-                                          SourceManager *SrcManager)
+                                              SourceManager *SrcManager)
 {
   SourceLocation StmtStartLoc = S->getLocStart();
 
+  if (StmtStartLoc.isMacroID()) {
+    StmtStartLoc = SrcManager->getFileLoc(StmtStartLoc);
+  }
+
   FileID FID;
   unsigned StartOffset = 
     getLocationOffsetAndFileID(StmtStartLoc, FID, SrcManager);
@@ -980,6 +1012,17 @@ bool RewriteUtils::getEntireDeclGroupStrAndRemove(DeclGroupRef DGR,
   SourceRange LastRange = LastD->getSourceRange();
   SourceLocation EndLoc = getEndLocationUntil(LastRange, ';');
 
+  // This isn't really good, but if EndLoc is invalid, what can we do? 
+  if (EndLoc.isInvalid()) {
+    unsigned Off = 0;
+    const char *StartBuf = SrcManager->getCharacterData(StartLoc);
+    while ((*StartBuf != '\n') && (*StartBuf != ';') && (*StartBuf != '\0')) {
+      Off++;
+      StartBuf++;
+    }
+    assert(Off && "Zero offset!");
+    EndLoc = StartLoc.getLocWithOffset(Off);
+  }
   getStringBetweenLocs(Str, StartLoc, EndLoc);
   return !TheRewriter->RemoveText(SourceRange(StartLoc, EndLoc));
 }
@@ -1138,6 +1181,15 @@ bool RewriteUtils::removeVarInitExpr(const VarDecl *VD)
   const Expr *Init = VD->getInit();
   SourceRange ExprRange = Init->getSourceRange();
   SourceLocation InitEndLoc = ExprRange.getEnd();
+  // handle macro, e.g.:
+  // #define NULL 0
+  // void foo(void)
+  // {
+    // int *p = NULL;
+  // }
+  if (SrcManager->isMacroBodyExpansion(InitEndLoc)) {
+    InitEndLoc = SrcManager->getFileLoc(InitEndLoc);
+  }
   return !TheRewriter->RemoveText(SourceRange(InitStartLoc, InitEndLoc));
 }
 
@@ -1260,7 +1312,8 @@ bool RewriteUtils::replaceUnionWithStruct(const NamedDecl *ND)
   std::string TmpStr(StartBuf, RangeSize);
   std::string UStr = "union";
   size_t Pos = TmpStr.find(UStr);
-  TransAssert((Pos != std::string::npos) && "Bad Name Position!");
+  if (Pos == std::string::npos)
+    return true;
 
   if (Pos != 0)
     StartLoc = StartLoc.getLocWithOffset(Pos);
@@ -1295,8 +1348,14 @@ bool RewriteUtils::removeArraySubscriptExpr(const Expr *E)
   StartLoc = StartLoc.getLocWithOffset(Offset);
 
   SourceLocation EndLoc = ERange.getEnd();
-  EndLoc = getLocationUntil(EndLoc, ']');
-  return !TheRewriter->RemoveText(SourceRange(StartLoc, EndLoc));
+  EndLoc = EndLoc.getLocWithOffset(1);
+  if (EndLoc.isInvalid())
+    return !TheRewriter->RemoveText(SourceRange(StartLoc, ERange.getEnd()));
+
+  SourceLocation RBLoc = getLocationUntil(EndLoc, ']');
+  if (RBLoc.isInvalid())
+    return !TheRewriter->RemoveText(SourceRange(StartLoc, EndLoc));
+  return !TheRewriter->RemoveText(SourceRange(StartLoc, RBLoc));
 }
 
 bool RewriteUtils::getFunctionDefStrAndRemove(const FunctionDecl *FD,
@@ -1330,6 +1389,14 @@ bool RewriteUtils::getFunctionDeclStrAndRemove(const FunctionDecl *FD,
   return !TheRewriter->RemoveText(SourceRange(StartLoc, EndLoc));
 }
 
+bool RewriteUtils::replaceFunctionDefWithStr(const FunctionDecl *FD,
+                                             const std::string &Str)
+{
+  const Stmt *Body = FD->getBody();
+  TransAssert(Body && "FunctionDecl is not a definition!");
+  return !TheRewriter->ReplaceText(Body->getSourceRange(), Str);
+}
+
 // FIXME: probably we don't need this function, because we could use 
 //        removeDecl insteadly
 bool RewriteUtils::removeFieldDecl(const FieldDecl *FD)
@@ -1549,6 +1616,18 @@ bool RewriteUtils::removeTextFromLeftAt(SourceRange Range, char C,
   return !TheRewriter->RemoveText(SourceRange(StartLoc, EndLoc));
 }
 
+SourceLocation RewriteUtils::getLocationFromLeftUntil(SourceLocation StartLoc,
+                                                      char C)
+{
+  const char *StartBuf = SrcManager->getCharacterData(StartLoc);
+  int Offset = 0;
+  while (*StartBuf != C) {
+    StartBuf--;
+    Offset--;
+  }
+  return StartLoc.getLocWithOffset(Offset);
+}
+
 bool RewriteUtils::removeTextUntil(SourceRange Range, char C)
 {
   SourceLocation StartLoc = Range.getBegin();
@@ -1590,7 +1669,10 @@ bool RewriteUtils::removeClassDecls(const CXXRecordDecl *CXXRD)
     SourceLocation LocEnd;
     if ((*I)->isThisDeclarationADefinition()) {
       LocEnd = (*I)->getRBraceLoc();
-      LocEnd = getLocationUntil(LocEnd, ';');
+      if (LocEnd.isValid())
+        LocEnd = getLocationUntil(LocEnd, ';');
+      else
+        LocEnd = getEndLocationUntil(Range, ';');
     }
     else {
       LocEnd = getEndLocationUntil(Range, ';');
