@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013 The University of Utah
+// Copyright (c) 2012, 2013, 2014 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -359,7 +359,7 @@ void SimpleInliner::doAnalysis(void)
     if (TransformationCounter == ValidInstanceNum) {
       // It's possible the direct callee is not a definition
       if (!CalleeDecl->isThisDeclarationADefinition()) {
-        CalleeDecl = CalleeDecl->getFirstDeclaration();
+        CalleeDecl = CalleeDecl->getCanonicalDecl();
         for(FunctionDecl::redecl_iterator RI = CalleeDecl->redecls_begin(),
             RE = CalleeDecl->redecls_end(); RI != RE; ++RI) {
           if ((*RI)->isThisDeclarationADefinition()) {
@@ -388,7 +388,7 @@ std::string SimpleInliner::getNewTmpName(void)
 
 void SimpleInliner::createReturnVar(void)
 {
-  const Type *FDType = CurrentFD->getResultType().getTypePtr();
+  const Type *FDType = CurrentFD->getReturnType().getTypePtr();
   const Type *CallExprType = TheCallExpr->getCallReturnType().getTypePtr();
 
   // We don't need tmp var
@@ -398,7 +398,7 @@ void SimpleInliner::createReturnVar(void)
 
   TmpVarName = getNewTmpName();
   std::string VarStr = TmpVarName;
-  CurrentFD->getResultType().getAsStringInternal(VarStr, 
+  CurrentFD->getReturnType().getAsStringInternal(VarStr, 
                                Context->getPrintingPolicy());
   VarStr += ";";
   RewriteHelper->addLocalVarToFunc(VarStr, TheCaller);
