@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012 The University of Utah
+// Copyright (c) 2012, 2013, 2014 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -15,6 +15,7 @@
 #include "SimplifyDependentTypedef.h"
 
 #include "clang/AST/RecursiveASTVisitor.h"
+#include "clang/AST/ASTContext.h"
 #include "clang/Basic/SourceManager.h"
 
 #include "TransformationManager.h"
@@ -61,8 +62,7 @@ class DependentTypedefTemplateTypeParmTypeVisitor : public
 public:
   explicit DependentTypedefTemplateTypeParmTypeVisitor(
              SimplifyDependentTypedef *Instance)
-    : ConsumerInstance(Instance),
-      IsValidType(false)
+    : IsValidType(false)
   { }
 
   bool VisitTemplateTypeParmType(TemplateTypeParmType *Ty);
@@ -83,8 +83,6 @@ private:
 
   TemplateTypeParmTypeSet *TypeSet;
 
-  SimplifyDependentTypedef *ConsumerInstance;
-
   bool IsValidType;
 };
 
@@ -220,9 +218,7 @@ void SimplifyDependentTypedef::handleOneTypedefDecl(const TypedefDecl *D)
 
 SimplifyDependentTypedef::~SimplifyDependentTypedef(void)
 {
-  if (CollectionVisitor)
-    delete CollectionVisitor;
-  if (TemplateTypeParmTypeVisitor)
-    delete TemplateTypeParmTypeVisitor;
+  delete CollectionVisitor;
+  delete TemplateTypeParmTypeVisitor;
 }
 
