@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013 The University of Utah
+// Copyright (c) 2012, 2013, 2014 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -52,7 +52,7 @@ private:
 
 bool SimplifyStructUnionDeclVisitor::VisitFunctionDecl(FunctionDecl *FD)
 {
-  const Type *T = FD->getResultType().getTypePtr();
+  const Type *T = FD->getReturnType().getTypePtr();
   return ConsumerInstance->handleOneDeclarator(T);
 }
 
@@ -182,7 +182,7 @@ bool SimplifyStructUnionDecl::isSafeToRemoveName(void)
     return false;
 
   const RecordDecl *RD = 
-    dyn_cast<RecordDecl>(TheRecordDecl->getFirstDeclaration());
+    dyn_cast<RecordDecl>(TheRecordDecl->getCanonicalDecl());
   RecordDecl::redecl_iterator I = RD->redecls_begin();
   RecordDecl::redecl_iterator E = RD->redecls_end();
   ++I;
