@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013 The University of Utah
+// Copyright (c) 2012, 2013, 2014 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -121,7 +121,8 @@ void Transformation::getTransErrorMsg(std::string &ErrorMsg)
     ErrorMsg = "Internal transformation error!";
   }
   else if (TransError == TransMaxInstanceError) {
-    ErrorMsg = "The counter value exceeded the number of transformation instances!";
+    ErrorMsg = 
+      "The counter value exceeded the number of transformation instances!";
   }
   else if (TransError == TransMaxVarsError) {
     ErrorMsg = "Too many variables!";
@@ -141,6 +142,10 @@ void Transformation::getTransErrorMsg(std::string &ErrorMsg)
   else if (TransError == TransNoTextModificationError) {
     ErrorMsg = "No modification to the transformed program!";
   }
+  else if (TransError == TransToCounterTooBigError) {
+    ErrorMsg = 
+      "The to-counter value exceeded the number of transformation instances!";
+  }
   else {
     TransAssert(0 && "Unknown transformation error!");
   }
@@ -498,7 +503,9 @@ const FunctionDecl *Transformation::lookupFunctionDeclInGlobal(
 }
 
 const FunctionDecl *Transformation::lookupFunctionDeclFromBases(
-        DeclarationName &DName, const CXXRecordDecl *CXXRD)
+        DeclarationName &DName, 
+        const CXXRecordDecl *CXXRD, 
+        DeclContextSet &VisitedCtxs)
 {
   for (CXXRecordDecl::base_class_const_iterator I =
        CXXRD->bases_begin(), E = CXXRD->bases_end(); I != E; ++I) {
@@ -513,15 +520,20 @@ const FunctionDecl *Transformation::lookupFunctionDeclFromBases(
     const CXXRecordDecl *BaseDef = Base->getDefinition();
     if (!BaseDef)
       continue;
-    if (const FunctionDecl *FD = lookupFunctionDecl(DName, BaseDef))
+    if (const FunctionDecl *FD = 
+        lookupFunctionDecl(DName, BaseDef, VisitedCtxs))
       return FD;
   }
   return NULL;
 }
 
 const FunctionDecl *Transformation::lookupFunctionDeclFromCtx(
-        DeclarationName &DName, const DeclContext *Ctx)
+        DeclarationName &DName, 
+        const DeclContext *Ctx,
+        DeclContextSet &VisitedCtxs)
 {
+  if (dyn_cast<LinkageSpecDecl>(Ctx))
+    return NULL;
   DeclContext::lookup_const_result Result = Ctx->lookup(DName);
   for (DeclContext::lookup_const_iterator I = Result.begin(), E = Result.end();
        I != E; ++I) {
@@ -545,7 +557,8 @@ const FunctionDecl *Transformation::lookupFunctionDeclFromCtx(
       const DeclContext *Ctx = getDeclContextFromSpecifier(NNS);
       if (!Ctx)
         continue;
-      if (const FunctionDecl *FD = lookupFunctionDecl(DName, Ctx))
+      if (const FunctionDecl *FD = 
+          lookupFunctionDecl(DName, Ctx, VisitedCtxs))
         return FD;
     }
   }
@@ -553,9 +566,18 @@ const FunctionDecl *Transformation::lookupFunctionDeclFromCtx(
 }
 
 const FunctionDecl *Transformation::lookupFunctionDecl(
-        DeclarationName &DName, const DeclContext *Ctx)
+        DeclarationName &DName, 
+        const DeclContext *Ctx, 
+        DeclContextSet &VisitedCtxs)
 {
-  if (const FunctionDecl *FD = lookupFunctionDeclFromCtx(DName, Ctx))
+  if (dyn_cast<LinkageSpecDecl>(Ctx))
+    return NULL;
+  if (VisitedCtxs.count(Ctx))
+    return NULL;
+  VisitedCtxs.insert(Ctx);
+
+  if (const FunctionDecl *FD = 
+      lookupFunctionDeclFromCtx(DName, Ctx, VisitedCtxs))
     return FD;
  
   // lookup base classes:
@@ -565,18 +587,18 @@ const FunctionDecl *Transformation::lookupFunctionDecl(
   if (Ctx->isRecord()) {
     const RecordDecl *RD = dyn_cast<RecordDecl>(Ctx);
     if (const CXXRecordDecl *CXXRD = dyn_cast<CXXRecordDecl>(RD)) {
-      if (const FunctionDecl *FD = lookupFunctionDeclFromBases(DName, CXXRD))
+      if (const FunctionDecl *FD = 
+          lookupFunctionDeclFromBases(DName, CXXRD, VisitedCtxs))
         return FD;
     }
   }
 
-  for (DeclContext::udir_iterator I = Ctx->using_directives_begin(),
-       E = Ctx->using_directives_end(); I != E; ++I) {
-    const NamespaceDecl *ND = (*I)->getNominatedNamespace();
+  for (auto *I : Ctx->using_directives()) {
+    const NamespaceDecl *ND = I->getNominatedNamespace();
     // avoid infinite recursion
     if (ND->getLookupParent() == Ctx)
       return NULL;
-    if (const FunctionDecl *FD = lookupFunctionDecl(DName, ND))
+    if (const FunctionDecl *FD = lookupFunctionDecl(DName, ND, VisitedCtxs))
       return FD;
   }
 
@@ -584,7 +606,7 @@ const FunctionDecl *Transformation::lookupFunctionDecl(
   if (!ParentCtx || dyn_cast<LinkageSpecDecl>(ParentCtx))
     return NULL;
 
-  return lookupFunctionDecl(DName, ParentCtx);
+  return lookupFunctionDecl(DName, ParentCtx, VisitedCtxs);
 }
 
 const DeclContext *Transformation::getDeclContextFromSpecifier(
