@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013 The University of Utah
+// Copyright (c) 2012, 2013, 2014 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -14,6 +14,7 @@
 #include <string>
 #include <cstdlib>
 #include <cassert>
+#include "llvm/ADT/SmallPtrSet.h"
 #include "clang/AST/ASTConsumer.h"
 #include "clang/Rewrite/Core/Rewriter.h"
 #include "RewriteUtils.h"
@@ -52,7 +53,8 @@ typedef enum {
   TransNoValidVarsError,
   TransNoValidFunsError,
   TransNoValidParamsError,
-  TransNoTextModificationError
+  TransNoTextModificationError,
+  TransToCounterTooBigError
 } TransformationError;
 
 class Transformation : public clang::ASTConsumer {
@@ -69,12 +71,34 @@ public:
       TransError(TransSuccess),
       DescriptionString(Desc),
       RewriteHelper(NULL),
-      Rewritten(false)
+      Rewritten(false),
+      MultipleRewrites(false),
+      ToCounter(-1)
   {
     // Nothing to do
   }
 
-  virtual ~Transformation(void);
+  Transformation(const char *TransName, 
+                 const char *Desc, 
+                 bool MultipleRewritesFlag)
+    : Name(TransName),
+      TransformationCounter(-1),
+      ValidInstanceNum(0),
+      QueryInstanceOnly(false),
+      Context(NULL),
+      SrcManager(NULL),
+      TransError(TransSuccess),
+      DescriptionString(Desc),
+      RewriteHelper(NULL),
+      Rewritten(false),
+      MultipleRewrites(MultipleRewritesFlag),
+      ToCounter(-1)
+  {
+    // Nothing to do
+  }
+
+
+  virtual ~Transformation();
 
   void outputOriginalSource(llvm::raw_ostream &OutStream);
 
@@ -84,33 +108,46 @@ public:
     TransformationCounter = Counter;
   }
 
+  void setToCounter(int Counter) {
+    ToCounter = Counter;
+  }
+
+  bool isMultipleRewritesEnabled() {
+    return MultipleRewrites;
+  }
+
   void setQueryInstanceFlag(bool Flag) {
     QueryInstanceOnly = Flag;
   }
 
-  bool transSuccess(void) {
+  bool transSuccess() {
     return (TransError == TransSuccess);
   }
   
-  bool transMaxInstanceError(void) {
+  bool transMaxInstanceError() {
     return (TransError == TransMaxInstanceError);
   }
 
-  bool transInternalError(void) {
+  bool transInternalError() {
     return (TransError == TransInternalError);
   }
 
-  std::string &getDescription(void) {
+  bool isInvalidCounterError() {
+    return ((TransError == TransMaxInstanceError) ||
+            (TransError == TransToCounterTooBigError));
+  }
+
+  std::string &getDescription() {
     return DescriptionString;
   }
 
   void getTransErrorMsg(std::string &ErrorMsg);
 
-  int getNumTransformationInstances(void) {
+  int getNumTransformationInstances() {
     return ValidInstanceNum;
   }
 
-  virtual bool skipCounter(void) {
+  virtual bool skipCounter() {
     return false;
   }
 
@@ -122,6 +159,8 @@ protected:
 
   typedef llvm::SmallVector<const clang::Expr *, 10> ExprVector;
 
+  typedef llvm::SmallPtrSet<const clang::DeclContext *, 20> DeclContextSet;
+
   unsigned int getArrayDimension(const clang::ArrayType *ArrayTy);
 
   unsigned int getArrayDimensionAndTypes(const clang::ArrayType *ArrayTy,
@@ -170,13 +209,19 @@ protected:
   bool isCXXMemberExpr(const clang::MemberExpr *ME);
 
   const clang::FunctionDecl *lookupFunctionDecl(
-          clang::DeclarationName &DName, const clang::DeclContext *Ctx);
+          clang::DeclarationName &DName, 
+          const clang::DeclContext *Ctx,
+          DeclContextSet &VisitedCtxs);
 
   const clang::FunctionDecl *lookupFunctionDeclFromCtx(
-          clang::DeclarationName &DName, const clang::DeclContext *Ctx);
+          clang::DeclarationName &DName, 
+          const clang::DeclContext *Ctx,
+          DeclContextSet &VisitedCtxs);
 
   const clang::FunctionDecl *lookupFunctionDeclFromBases(
-          clang::DeclarationName &DName, const clang::CXXRecordDecl *CXXRD);
+          clang::DeclarationName &DName, 
+          const clang::CXXRecordDecl *CXXRD,
+          DeclContextSet &VisitedCtxs);
 
   const clang::FunctionDecl *lookupFunctionDeclInGlobal(
           clang::DeclarationName &DName, const clang::DeclContext *Ctx);
@@ -246,6 +291,10 @@ protected:
   RewriteUtils *RewriteHelper;
 
   bool Rewritten;
+
+  const bool MultipleRewrites;
+
+  int ToCounter;
 };
 
 class TransNameQueryVisitor;
@@ -256,9 +305,9 @@ friend class TransNameQueryVisitor;
 public:
   explicit TransNameQueryWrap(const std::string &Prefix);
 
-  ~TransNameQueryWrap(void);
+  ~TransNameQueryWrap();
 
-  unsigned int getMaxNamePostfix(void) {
+  unsigned int getMaxNamePostfix() {
     return MaxPostfix;
   }
 
@@ -273,7 +322,7 @@ private:
   TransNameQueryVisitor *NameQueryVisitor;
 
   // Unimplemented
-  TransNameQueryWrap(void);
+  TransNameQueryWrap();
 
   TransNameQueryWrap(const TransNameQueryWrap &);
 
