@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013 The University of Utah
+// Copyright (c) 2012, 2013, 2014 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -41,12 +41,14 @@
 
 using namespace clang;
 
+int TransformationManager::ErrorInvalidCounter = 1;
+
 TransformationManager* TransformationManager::Instance;
 
 std::map<std::string, Transformation *> *
 TransformationManager::TransformationsMapPtr;
 
-TransformationManager *TransformationManager::GetInstance(void)
+TransformationManager *TransformationManager::GetInstance()
 {
   if (TransformationManager::Instance)
     return TransformationManager::Instance;
@@ -59,7 +61,7 @@ TransformationManager *TransformationManager::GetInstance(void)
   return TransformationManager::Instance;
 }
 
-bool TransformationManager::isCXXLangOpt(void)
+bool TransformationManager::isCXXLangOpt()
 {
   TransAssert(TransformationManager::Instance && "Invalid Instance!");
   TransAssert(TransformationManager::Instance->ClangInstance && 
@@ -68,7 +70,7 @@ bool TransformationManager::isCXXLangOpt(void)
           .CPlusPlus);
 }
 
-bool TransformationManager::isCLangOpt(void)
+bool TransformationManager::isCLangOpt()
 {
   TransAssert(TransformationManager::Instance && "Invalid Instance!");
   TransAssert(TransformationManager::Instance->ClangInstance && 
@@ -110,11 +112,11 @@ bool TransformationManager::initializeCompilerInstance(std::string &ErrorMsg)
   TargetOpts.Triple = LLVM_DEFAULT_TARGET_TRIPLE;
   TargetInfo *Target = 
     TargetInfo::CreateTargetInfo(ClangInstance->getDiagnostics(),
-                                 &TargetOpts);
+                                 ClangInstance->getInvocation().TargetOpts);
   ClangInstance->setTarget(Target);
   ClangInstance->createFileManager();
   ClangInstance->createSourceManager(ClangInstance->getFileManager());
-  ClangInstance->createPreprocessor();
+  ClangInstance->createPreprocessor(TU_Complete);
 
   DiagnosticConsumer &DgClient = ClangInstance->getDiagnosticClient();
   DgClient.BeginSourceFile(ClangInstance->getLangOpts(),
@@ -135,7 +137,7 @@ bool TransformationManager::initializeCompilerInstance(std::string &ErrorMsg)
   return true;
 }
 
-void TransformationManager::Finalize(void)
+void TransformationManager::Finalize()
 {
   assert(TransformationManager::Instance);
   
@@ -156,15 +158,15 @@ void TransformationManager::Finalize(void)
   Instance = NULL;
 }
 
-llvm::raw_ostream *TransformationManager::getOutStream(void)
+llvm::raw_ostream *TransformationManager::getOutStream()
 {
   if (OutputFileName.empty())
     return &(llvm::outs());
 
-  std::string Err;
-  llvm::raw_fd_ostream *Out = 
-    new llvm::raw_fd_ostream(OutputFileName.c_str(), Err);
-  assert(Err.empty() && "Cannot open output file!");
+  std::string ES;
+  llvm::raw_fd_ostream *Out = new llvm::raw_fd_ostream(
+      OutputFileName.c_str(), ES, llvm::sys::fs::F_RW);
+  assert(ES == "" && "Cannot open output file!");
   return Out;
 }
 
@@ -174,15 +176,28 @@ void TransformationManager::closeOutStream(llvm::raw_ostream *OutStream)
     delete OutStream;
 }
 
-bool TransformationManager::doTransformation(std::string &ErrorMsg)
+bool TransformationManager::doTransformation(std::string &ErrorMsg, int &ErrorCode)
 {
   ErrorMsg = "";
 
   ClangInstance->createSema(TU_Complete, 0);
-  ClangInstance->getDiagnostics().setSuppressAllDiagnostics(true);
+  DiagnosticsEngine &Diag = ClangInstance->getDiagnostics();
+  Diag.setSuppressAllDiagnostics(true);
+  Diag.setIgnoreAllWarnings(true);
 
   CurrentTransformationImpl->setQueryInstanceFlag(QueryInstanceOnly);
   CurrentTransformationImpl->setTransformationCounter(TransformationCounter);
+  if (ToCounter > 0) {
+    if (CurrentTransformationImpl->isMultipleRewritesEnabled()) {
+      CurrentTransformationImpl->setToCounter(ToCounter);
+    }
+    else {
+      ErrorMsg = "current transformation[";
+      ErrorMsg += CurrentTransName; 
+      ErrorMsg += "] does not support multiple rewrites!";
+      return false;
+    }
+  }
 
   ParseAST(ClangInstance->getSema());
 
@@ -204,22 +219,33 @@ bool TransformationManager::doTransformation(std::string &ErrorMsg)
   }
   else {
     CurrentTransformationImpl->getTransErrorMsg(ErrorMsg);
+    if (CurrentTransformationImpl->isInvalidCounterError())
+      ErrorCode = ErrorInvalidCounter;
     RV = false;
   }
   closeOutStream(OutStream);
   return RV;
 }
 
-bool TransformationManager::verify(std::string &ErrorMsg)
+bool TransformationManager::verify(std::string &ErrorMsg, int &ErrorCode)
 {
   if (!CurrentTransformationImpl) {
     ErrorMsg = "Empty transformation instance!";
     return false;
   }
 
-  if ((TransformationCounter <= 0) && 
-      !CurrentTransformationImpl->skipCounter()) {
+  if (CurrentTransformationImpl->skipCounter())
+    return true;
+
+  if (TransformationCounter <= 0) {
     ErrorMsg = "Invalid transformation counter!";
+    ErrorCode = ErrorInvalidCounter;
+    return false;
+  }
+
+  if ((ToCounter > 0) && (ToCounter < TransformationCounter)) {
+    ErrorMsg = "to-counter value cannot be smaller than counter value!";
+    ErrorCode = ErrorInvalidCounter;
     return false;
   }
 
@@ -242,7 +268,7 @@ void TransformationManager::registerTransformation(
   (*TransformationManager::TransformationsMapPtr)[TransName] = TransImpl;
 }
 
-void TransformationManager::printTransformations(void)
+void TransformationManager::printTransformations()
 {
   llvm::outs() << "Registered Transformations:\n";
 
@@ -255,7 +281,7 @@ void TransformationManager::printTransformations(void)
   }
 }
 
-void TransformationManager::printTransformationNames(void)
+void TransformationManager::printTransformationNames()
 {
   std::map<std::string, Transformation *>::iterator I, E;
   for (I = TransformationsMap.begin(), 
@@ -265,7 +291,7 @@ void TransformationManager::printTransformationNames(void)
   }
 }
 
-void TransformationManager::outputNumTransformationInstances(void)
+void TransformationManager::outputNumTransformationInstances()
 {
   int NumInstances = 
     CurrentTransformationImpl->getNumTransformationInstances();
@@ -273,18 +299,20 @@ void TransformationManager::outputNumTransformationInstances(void)
                << NumInstances << "\n";
 }
 
-TransformationManager::TransformationManager(void)
+TransformationManager::TransformationManager()
   : CurrentTransformationImpl(NULL),
     TransformationCounter(-1),
+    ToCounter(-1),
     SrcFileName(""),
     OutputFileName(""),
+    CurrentTransName(""),
     ClangInstance(NULL),
     QueryInstanceOnly(false)
 {
   // Nothing to do
 }
 
-TransformationManager::~TransformationManager(void)
+TransformationManager::~TransformationManager()
 {
   // Nothing to do
 }
