@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012 The University of Utah
+// Copyright (c) 2012, 2013, 2014 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -85,14 +85,18 @@ void UnifyFunctionDecl::doAnalysis(void)
     const FunctionDecl *FDDef = NULL; 
     const FunctionDecl *FDDecl = NULL;
     const FunctionDecl *CanonicalFD = (*I);
-    const FunctionDecl *FirstFD = CanonicalFD->getFirstDeclaration();
+    const FunctionDecl *FirstFD = CanonicalFD->getCanonicalDecl();
 
     FunctionDecl::redecl_iterator RI = FirstFD->redecls_begin();
+    if (FirstFD->getSourceRange().isInvalid())
+      continue;
     ++RI;
     if (RI == FirstFD->redecls_end())
       continue;
 
     const FunctionDecl *SecondFD = (*RI);
+    if (SecondFD->getSourceRange().isInvalid())
+      continue;
     if (FirstFD->isThisDeclarationADefinition()) {
       FDDef = FirstFD;
       TransAssert(!SecondFD->isThisDeclarationADefinition() &&
