@@ -1,6 +1,6 @@
 //===----------------------------------------------------------------------===//
 //
-// Copyright (c) 2012, 2013 The University of Utah
+// Copyright (c) 2012, 2013, 2014 The University of Utah
 // All rights reserved.
 //
 // This file is distributed under the University of Illinois Open Source
@@ -67,7 +67,7 @@ private:
 
 bool UnionToStructCollectionVisitor::VisitFunctionDecl(FunctionDecl *FD)
 {
-  const Type *T = FD->getResultType().getTypePtr();
+  const Type *T = FD->getReturnType().getTypePtr();
   ConsumerInstance->addOneDeclarator(FD, T);
   return true;
 }
@@ -185,7 +185,7 @@ void UnionToStruct::rewriteOneRecordDecl(const RecordDecl *RD)
 void UnionToStruct::rewriteRecordDecls(void)
 {
   const RecordDecl *RD = 
-    dyn_cast<RecordDecl>(TheRecordDecl->getFirstDeclaration());
+    dyn_cast<RecordDecl>(TheRecordDecl->getCanonicalDecl());
   TransAssert(RD && "NULL RecordDecl!");
   for(RecordDecl::redecl_iterator I = RD->redecls_begin(),
       E = RD->redecls_end(); I != E; ++I) {
@@ -327,12 +327,18 @@ void UnionToStruct::rewriteOneVarDecl(const VarDecl *VD)
     return;
   }
   
+  const Expr *IE = VD->getInit();
+  // Looks like we are safe to skip this
+  // e.g. 
+  // union U { int x; };
+  // void foo() { U y; }
+  if (dyn_cast<CXXConstructExpr>(IE))
+    return;
   if (!isValidRecordDecl(TheRecordDecl)) {
     RewriteHelper->removeVarInitExpr(VD);
     return;
   }
 
-  const Expr *IE = VD->getInit();
   const InitListExpr *ILE = dyn_cast<InitListExpr>(IE);
   TransAssert(ILE && "Bad InitListExpr!");
 
@@ -396,6 +402,8 @@ void UnionToStruct::addOneDeclarator(const DeclaratorDecl *DD, const Type *T)
 
   const RecordType *RDTy = T->getAsUnionType();
   TransAssert(RDTy && "Bad RecordType!");
+  if (RDTy->isIncompleteType())
+    return;
 
   const RecordDecl *RD = RDTy->getDecl(); 
   const RecordDecl *CanonicalRD = 
