@@ -1,6 +1,6 @@
 ## -*- mode: Perl -*-
 ##
-## Copyright (c) 2012, 2013 The University of Utah
+## Copyright (c) 2012, 2013, 2014 The University of Utah
 ## All rights reserved.
 ##
 ## This file is distributed under the University of Illinois Open Source
@@ -30,7 +30,7 @@ my $ORIG_DIR;
 sub check_prereqs () {
     $ORIG_DIR = getcwd();
     my $path;
-    if ($FindBin::Bin eq bindir) {
+    if ($FindBin::RealBin eq bindir) {
 	# This script is in the installation directory.
 	# Use the installed `clang_delta'.
 	$path = libexecdir . "/clang_delta";
@@ -64,34 +64,41 @@ sub transform ($$$) {
     my $tmpfile = POSIX::tmpnam();
     my $cmd = "$clang_delta --transformation=$which --counter=$index $cfile";
     print "$cmd\n" if $VERBOSE;
-    my $res = runit ("$cmd > $tmpfile");
+    my $res = run_clang_delta ("$cmd > $tmpfile");
     if ($res==0) {
 	system "mv $tmpfile $cfile";
 	return ($OK, \$index);
     } else {
-	if ($res == -1) {
+	if (($res == -1) || ($res == -2)) {
 	} else {
-	    my $crashfile = $tmpfile;
-	    $crashfile =~ s/\//_/g;
-	    my ($suffix) = $cfile =~ /(\.[^.]+)$/;
-	    $crashfile = "clang_delta_crash" . $crashfile . $suffix;
-	    my $crashfile_path = File::Spec->join($ORIG_DIR, $crashfile);
-	    File::Copy::copy($cfile, $crashfile_path);
-	    open TMPF, ">>$crashfile_path";
-	    print TMPF "\n\n";
-	    print TMPF "\/\/ this should reproduce the crash:\n";
-	    print TMPF "\/\/ $clang_delta --transformation=$which --counter=$index $crashfile_path\n";
-	    close TMPF;
-	    print "\n\n=======================================\n\n";
-	    print "OOPS: clang_delta crashed; please consider mailing\n";
-	    print "${crashfile}\n";
-	    print "to creduce-bugs\@flux.utah.edu and we will try to fix the bug\n";
-	    print "please also let us know what version of C-Reduce you are using\n";
-	    print "\n=======================================\n\n";
-	}
-	system "rm $tmpfile";
-	return ($STOP, \$index);
-    }    
+            my $n = int(rand(1000000));
+            my $crashfile = File::Spec->join($ORIG_DIR, "creduce_bug_$n");
+            File::Copy::copy($cfile, $crashfile) or die;
+            open  CRASH, ">>$crashfile";
+            print CRASH "\n\n";
+            print CRASH "\/\/ this should reproduce the crash:\n";
+            print CRASH "\/\/ $clang_delta --transformation=$which --counter=$index $crashfile\n";
+            close CRASH;
+            print <<"EOT";
+
+
+=======================================
+
+OOPS: clang_delta::$which has crashed, which means
+you have encountered a bug in C-Reduce. Please
+consider mailing ${crashfile} to
+creduce-bugs\@flux.utah.edu and we will try to fix
+the bug. Please also let us know what version of
+C-Reduce you are using and include any other
+details that may help us reproduce the problem.
+
+=======================================
+
+EOT
+        }
+        system "rm $tmpfile";
+        return ($STOP, \$index);
+    }
 }
 
 1;
